@@ -107,7 +107,7 @@ const char *strerror(int e)
 #endif
 
 #ifndef HAVE_UTIMES
-int utimes(char *filename, struct timeval *tvp)
+int utimes(const char *filename, struct timeval *tvp)
 {
 	struct utimbuf ub;
 
@@ -412,6 +412,14 @@ getsid(pid_t pid)
 }
 #endif
 
+#ifndef HAVE_KILLPG
+int
+killpg(pid_t pgrp, int sig)
+{
+	return kill(pgrp, sig);
+}
+#endif
+
 #ifdef FFLUSH_NULL_BUG
 #undef fflush
 int _ssh_compat_fflush(FILE *f)
