@@ -1,4 +1,4 @@
-/* $OpenBSD: sftp-client.h,v 1.34 2021/08/09 23:47:44 djm Exp $ */
+/* $OpenBSD: sftp-client.h,v 1.36 2022/03/31 03:07:03 djm Exp $ */
 
 /*
  * Copyright (c) 2001-2004 Damien Miller <djm@openbsd.org>
@@ -125,6 +125,9 @@ int do_statvfs(struct sftp_conn *, const char *, struct sftp_statvfs *, int);
 /* Rename 'oldpath' to 'newpath' */
 int do_rename(struct sftp_conn *, const char *, const char *, int);
 
+/* Copy 'oldpath' to 'newpath' */
+int do_copy(struct sftp_conn *, const char *, const char *);
+
 /* Link 'oldpath' to 'newpath' */
 int do_hardlink(struct sftp_conn *, const char *, const char *);
 
@@ -183,7 +186,7 @@ int crossload_dir(struct sftp_conn *from, struct sftp_conn *to,
 char *path_append(const char *, const char *);
 
 /* Make absolute path if relative path and CWD is given. Does not modify
- * original if the the path is already absolute. */
+ * original if the path is already absolute. */
 char *make_absolute(char *, const char *);
 
 /* Check if remote path is directory */
